@@ -16,7 +16,7 @@ use XML::LibXML::XPathContext;
 use Net::DAV::LockManager ();
 use Net::DAV::LockManager::DB ();
 
-our $VERSION = '1.302';
+our $VERSION = '1.305';
 $VERSION = eval $VERSION;  # convert development version into a simpler version number.
 
 our %implemented = (
@@ -66,7 +66,7 @@ sub run {
     my $fs = $self->filesys || die 'Filesys missing';
 
     my $method = $request->method;
-    my $path   = decode_utf8 uri_unescape $request->uri->path;
+    my $path   = uri_unescape $request->uri->path;
 
     if ( !defined $response ) {
         $response = HTTP::Response->new;
@@ -105,7 +105,7 @@ sub options {
 
 sub head {
     my ( $self, $request, $response ) = @_;
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
     my $fs   = $self->filesys;
 
     if ( $fs->test( 'f', $path ) && $fs->test( 'r', $path ) ) {
@@ -122,7 +122,7 @@ sub head {
 
 sub get {
     my ( $self, $request, $response ) = @_;
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
     my $fs   = $self->filesys;
 
     if ( $fs->test( 'f', $path ) && $fs->test( 'r', $path ) ) {
@@ -272,7 +272,7 @@ sub _parse_lock_header {
     my ($req)   = @_;
     my $depth   = $req->header('Depth');
     my %lockreq = (
-        'path' => decode_utf8( URI::Escape::uri_unescape( $req->uri->path ) ),
+        'path' => uri_unescape( $req->uri->path ),
 
         # Assuming basic auth for now.
         'user' => ( $req->authorization_basic() )[0] || '',
@@ -370,7 +370,7 @@ sub _active_lock_prop {
 
 sub unlock {
     my ( $self, $request, $response ) = @_;
-    my $path    = decode_utf8( URI::Escape::uri_unescape( $request->uri->path ) );
+    my $path    = uri_unescape( $request->uri->path );
     my $lockreq = _parse_lock_header($request);
 
     # No lock token supplied, we cannot unlock
@@ -428,7 +428,7 @@ sub put {
         return HTTP::Response->new( 403, 'Forbidden' );
     }
 
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
     my $fs   = $self->filesys;
 
     return HTTP::Response->new( 405, 'Method Not Allowed' ) if $fs->test( 'd', $path );
@@ -471,7 +471,7 @@ sub delete {
         return HTTP::Response->new( 404, 'Not Found', $response->headers );
     }
 
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
     my $fs   = $self->filesys;
     unless ( $fs->test( 'e', $path ) ) {
         return HTTP::Response->new( 404, 'Not Found', $response->headers );
@@ -479,11 +479,8 @@ sub delete {
 
     my $dom = XML::LibXML::Document->new( '1.0', 'utf-8' );
     my @error;
-    foreach my $part (
-        grep { $_ !~ m{/\.\.?$} }
-        map { s{/+}{/}g; $_ }
-        File::Find::Rule::Filesys::Virtual->virtual($fs)->in($path), $path
-      ) {
+    # see rt 46865: files first since rmdir() only removed empty directories
+    foreach my $part ( _get_files($fs, $path), _get_dirs($fs, $path), $path ) {
 
         next unless $fs->test( 'e', $part );
 
@@ -514,11 +511,12 @@ sub delete {
 
 sub copy {
     my ( $self, $request, $response ) = @_;
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
+    $path =~ s{/+$}{}; # see rt 46865
 
     # need to modify request to pay attention to destination address.
     my $lockreq = _parse_lock_header( $request );
-    $lockreq->{'path'} = decode_utf8( URI::Escape::uri_unescape( $request->header( 'Destination' ) ) );
+    $lockreq->{'path'} = uri_unescape( $request->header( 'Destination' ) );
     if ( !$self->_lock_manager()->can_modify( $lockreq ) ) {
         return HTTP::Response->new( 403, 'Forbidden' );
     }
@@ -526,26 +524,19 @@ sub copy {
 
     my $destination = $request->header('Destination');
     $destination = URI->new($destination)->path;
-    my $depth     = $request->header('Depth')     || 0;
+    $destination =~ s{/+$}{}; # see rt 46865
+
+    my $depth     = $request->header('Depth');
+    $depth = '' if !defined $depth;
+
     my $overwrite = $request->header('Overwrite') || 'F';
 
     if ( $fs->test( "f", $path ) ) {
         return $self->_copy_file( $request, $response );
     }
 
-    # it's a good approximation
-    $depth = 100 if defined $depth && $depth eq 'infinity';
-
-    my @files =
-      map { s{/+}{/}g; $_ }
-      File::Find::Rule::Filesys::Virtual->virtual($fs)->file->maxdepth($depth)
-      ->in($path);
-
-    my @dirs = reverse sort
-      grep { $_ !~ m{/\.\.?$} }
-      map { s{/+}{/}g; $_ }
-      File::Find::Rule::Filesys::Virtual->virtual($fs)->directory->maxdepth($depth)
-      ->in($path);
+    my @files = _get_files($fs, $path, $depth);
+    my @dirs  = _get_dirs($fs, $path, $depth);
 
     push @dirs, $path;
     foreach my $dir ( sort @dirs ) {
@@ -586,7 +577,7 @@ sub copy {
 
 sub _copy_file {
     my ( $self, $request, $response ) = @_;
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
     my $fs   = $self->filesys;
 
     my $destination = $request->header('Destination');
@@ -636,7 +627,7 @@ sub move {
     if ( !$self->_lock_manager()->can_modify( $lockreq ) ) {
         return HTTP::Response->new( 403, 'Forbidden' );
     }
-    $lockreq->{'path'} = decode_utf8( URI::Escape::uri_unescape( $request->header( 'Destination' ) ) );
+    $lockreq->{'path'} = uri_unescape( $request->header( 'Destination' ) );
     if ( !$self->_lock_manager()->can_modify( $lockreq ) ) {
         return HTTP::Response->new( 403, 'Forbidden' );
     }
@@ -656,7 +647,7 @@ sub move {
 
 sub mkcol {
     my ( $self, $request, $response ) = @_;
-    my $path = decode_utf8 uri_unescape $request->uri->path;
+    my $path = uri_unescape $request->uri->path;
 
     if ( !$self->_can_modify($request) ) {
         return HTTP::Response->new( 403, 'Forbidden' );
@@ -681,7 +672,7 @@ sub mkcol {
 
 sub propfind {
     my ( $self, $request, $response ) = @_;
-    my $path  = decode_utf8 uri_unescape $request->uri->path;
+    my $path  = uri_unescape $request->uri->path;
     my $fs    = $self->filesys;
     my $depth = $request->header('Depth');
 
@@ -752,7 +743,7 @@ sub propfind {
         my $is_dir = $fs->test( 'd', $path );
         my $resp = _dav_child( $multistat, 'response' );
         my $href = File::Spec->catdir(
-                map { uri_escape encode_utf8 $_} File::Spec->splitdir($path)
+                map { uri_escape $_} File::Spec->splitdir($path)
             ) . ( $is_dir && $path !~ m{/$} ? '/' : '');
         $href =~ tr{\\}{/};  # Protection from wrong slashes under Windows.
         _dav_child( $resp, 'href', $href );
@@ -792,6 +783,9 @@ sub propfind {
                         $prop->addChild( $active );
                     }
                 }
+                elsif ( $ns eq 'DAV:' && $name eq 'supportedlock' ) {
+                    $prop = _supportedlock_child( $okprops );
+                }
                 else {
                     my $prefix = $prefixes{$ns};
                     if ( !defined $prefix ) {
@@ -822,16 +816,7 @@ sub propfind {
             _dav_child( $okprops, 'getcontentlength', $is_dir ? () : ($size) );
             _dav_child( $okprops, 'getcontenttype', $is_dir ? 'httpd/unix-directory' : 'httpd/unix-file' );
             _dav_child( $okprops, 'getlastmodified', $mtime );
-            do {
-                $prop = _dav_child( $okprops, 'supportedlock' );
-                #for my $n (qw(exclusive shared)) {  # shared is currently not supported.
-                for my $n (qw(exclusive)) {
-                    my $lock = _dav_child( $prop, 'lockentry' );
-
-                    _dav_child( _dav_child( $lock, 'lockscope' ), $n );
-                    _dav_child( _dav_child( $lock, 'locktype' ), 'write' );
-                }
-            };
+            $prop = _supportedlock_child( $okprops );
             my $user = ($request->authorization_basic())[0]||'';
             my @locks = $self->_lock_manager()->list_all_locks({ 'path' => $path, 'user' => $user });
             if ( @locks ) {
@@ -866,6 +851,40 @@ sub propfind {
     return $response;
 }
 
+sub _supportedlock_child {
+    my ($okprops) = @_;
+    my $prop = _dav_child( $okprops, 'supportedlock' );
+    #for my $n (qw(exclusive shared)) {  # shared is currently not supported.
+    for my $n (qw(exclusive)) {
+        my $lock = _dav_child( $prop, 'lockentry' );
+
+        _dav_child( _dav_child( $lock, 'lockscope' ), $n );
+        _dav_child( _dav_child( $lock, 'locktype' ), 'write' );
+    }
+
+    return $prop;
+}
+
+sub _get_files {
+    my ($fs, $path, $depth) = @_;
+    reverse map { s{/+}{/}g;s{/$}{}; $_ }
+    (defined $depth && $depth =~ m{\A\d+\z}) ?
+      File::Find::Rule::Filesys::Virtual->virtual($fs)->file->maxdepth($depth)->in($path)
+      : File::Find::Rule::Filesys::Virtual->virtual($fs)->file->in($path)
+      ;
+}
+
+sub _get_dirs {
+    my ($fs, $path, $depth) = @_;
+    return reverse sort
+    grep { $_ !~ m{/\.\.?$} }
+    map { s{/+}{/}g;s{/$}{}; $_ }
+    (defined $depth && $depth =~ m{\A\d+\z}) ?
+       File::Find::Rule::Filesys::Virtual->virtual($fs)->directory->maxdepth($depth)->in($path)
+       : File::Find::Rule::Filesys::Virtual->virtual($fs)->directory->in($path)
+       ;
+}
+
 1;
 
 __END__
